@@ -19,7 +19,7 @@
 
 cd "$(dirname $0)/html-converters/"
 case "$(printf '%s' "$1" | tr '[:upper:]' '[:lower:]')" in
-	*.md|*.mkd) exec ./md2html; ;;
+	*.markdown|*.mdown|*.md|*.mkd) exec ./md2html; ;;
 	*.rst) exec ./rst2html; ;;
 	*.[1-9]) exec ./man2html; ;;
 	*.htm|*.html) exec cat; ;;
